

;--- Compiler configuration:
#option Explicit

;--- Config serial port for S.Port-Telemetry once:
;    (Priority lower than serial library; so softserial1.h
;     can overwrite port-wide settings to input.)
#startup SPortInit, 60


Sub SPortInit()

  ;--- Config Serial UART channel 2 for S.Port telemetry protocol
  ;--- (half duplex receiving and sending on same port):
  #define SER2_BAUD 57600            ; 57600 baud
  #define SER2_DATABITS 8            ; 8 databits
  #define SER2_STOPBITS 1            ; 1 stopbit
  #define SER2_INVERT On 'SER2_INVERT   ; polarity is set im main file depending if Jumper receiver or FrSky/Radiomaster
  #define SER2_RXNOWAIT Off          ; wait for stopbit (blocking receive)
  ; Config I/O ports for receiving/sending (halpduplex) S.Port-Data:
  #define SER2_RXPORT RECEIVER_PORT  ; user config (see top of this file)
  #define SER2_RXPIN  RECEIVER_PIN   ;
  #define SER2_TXPORT RECEIVER_PORT  ;
  #define SER2_TXPIN  RECEIVER_PIN   ;
  ; Set port to input by default. Port ist set to high by "softserial.h",
  ; so internal pullup is activated. Port will be set to output before
  ; sending and set back to input afterwards by program.
  Dir RECEIVER_PORT.RECEIVER_PIN In

End Sub


Sub SPortSend()
; Sends one S.Port dataframe and calculates checksum:

  Dim bByteNr As Byte
  Dim wCrc8S As Word

  ;Set pin direction to output and activate stopbit-level:
  Dir SER2_RXPORT.SER2_RXPIN Out
  #if SER2_INVERT = Off             'Jumper R8 receiver
    Set SER2_RXPORT.SER2_RXPIN On
  #endif
  #if SER2_INVERT = On              'FrSky receiver
    Set SER2_RXPORT.SER2_RXPIN Off
  #endif

  ;Send 8-byte sPort dataframe (can be up to 10 byte due to bytestuffing):
  wCrc8S = 0
  For bByteNr = 1 To 7
    ;Calculate checksum on the fly:
    wCrc8S = wCrc8S + SPortFrame(bByteNr)
    wCrc8S = (wCrc8S + wCrc8S / 0x100) AND 0x00FF '8bit shift right
    ;Send data byte:
    ByteStuffedSend(SPortFrame(bByteNr))
  Next bByteNr
  ;Calculate checksum byte:
  SPortFrame(8) = 0x00FF - wCrc8S
  ;Send checksum byte as byte 8:
  ByteStuffedSend(SPortFrame(8))

  ;Set pin direction back to input and activate stopbit-level:
  Dir SER2_RXPORT.SER2_RXPIN In
  #if SER2_INVERT = Off             'Jumper R8 receiver
    Set SER2_RXPORT.SER2_RXPIN On
  #endif
  #if SER2_INVERT = On              'FrSky receiver
    Set SER2_RXPORT.SER2_RXPIN Off
  #endif

End Sub


Sub ByteStuffedSend(In bSendByte As Byte)
;Replace control characters (byte-stuffing) and send one byte to S.Port:

  Select Case bSendByte
    Case 0x7E
      Ser2Send 0x7D
      Ser2Send 0x5E
    Case 0x7D
      Ser2Send 0x7D
      Ser2Send 0x5D
    Case Else
      Ser2Send bSendByte
  End Select

End Sub
