

;--- Compiler configuration:
#option Explicit

;--- Config serial port for iBus telemetry once:
;    (Priority higher than serial library; otherwise softserial2.h
;     sets port.pin to output since is Tx on the same pin.)
#startup iBusPortInit, 110


Sub iBusPortInit

  ;--- Config Serial UART channel 2 for iBus telemetry protocol
  ;--- (half duplex receiving and sending on same port):
  #define SER2_BAUD 115200           ; 115200 baud
  #define SER2_DATABITS 8            ; 8 databits
  #define SER2_STOPBITS 1            ; 1 stopbit
  #define SER2_INVERT Off            ; ttl polarity
  #define SER2_RXNOWAIT Off          ; wait for stopbit (blocking receive)
  ; Config I/O ports for receiving/sending (halpduplex) iBus-Data:
  #define SER2_RXPORT RECEIVER_PORT  ; user config (see top of this file)
  #define SER2_RXPIN  RECEIVER_PIN   ;
  #define SER2_TXPORT RECEIVER_PORT  ;
  #define SER2_TXPIN  RECEIVER_PIN   ;

  ; Port.Pin is input by default, but set to output and high by
  ; "softserial.h" because Rx and Tx is the same line here.
  ; Port must be input by default and will be set to output
  ; before sending and set back to input afterwards by program.
  Dir RECEIVER_PORT.RECEIVER_PIN In

End Sub


Sub iBusRegisterIDs()
;First time registering of all iBus IDs.

  Dim bIDNr As Byte

  ;-- Wait for free iBus-ID (0x81 to 0x8F):
  Do
    iBusReceive()
  Loop Until (iBusFrame(2) > 0x80) And (iBusFrame(2) < 0x90)

  ;-- Calculate the other IDs by simply incrementing the addresses:
  iBusID(1) = iBusFrame(2)+0x20
  For  bIDNr = 2 To bNrOfSensors
    iBusID(bIDNr) = iBusID(bIDNr-1) + 1
    If iBusID(bIDNr) = 0xAF Then  'break at last possible address
       bNrOfSensors = bIDNr
       Exit For
    End If
  Next bIDNr

End Sub


Sub iBusReceive
;Receives one iBus dataframe:
; (don't remove the checksum calculation - the time ist needed
;  as delay for the port to get ready for sending !)

  Dim bByteNr, bIBusError As Byte
  Dim wChecksum As Word

  ;--- Loop until valid frame received:
  Do

    wChecksum = 0xFFFF
    bIBusError = 255

    ;-- Make shure, that we don't jump inside a dataframe:
    ;   (255 loops need 207 us, one byte (8Bit + startbit + stopbit) at 115200 baud need 87 us).
    Do
      bIBusError -= 1
      If RECEIVER_PORT.RECEIVER_PIN = 0 Then bIBusError = 255  'Restart loop, if a startbit is detected
    Loop Until bIBusError = 0

    ;-- Receive first byte which is the length of frame:
    iBusFrame(1) = Ser2Receive
    ;-- Receive the other bytes if value if first byte is valid:
    If iBusFrame(1) = 4 Then  '(iBusFrame(1) < 7) Then 'AND (iBusFrame(1) > 3) Then
        ;- Receive from byte 2 to end of frame
        For bByteNr = 2 to iBusFrame(1)
          iBusFrame(bByteNr) = Ser2Receive
        Next bByteNr
        ;- Calculate checksum
        For bByteNr = 1 to iBusFrame(1) - 2  'last two bytes are the received checksum
          wChecksum -= iBusFrame(bByteNr)    'calculated checksum
        Next bByteNr
        ;- Do some plausibility tests and set errorflags:
        bByteNr = iBusFrame(1) 'bByteNr is frame-length now
        If [Byte]wChecksum <> iBusFrame(bByteNr-1) Then bIBusError.0 = 1  '1
        If wChecksum_H <> iBusFrame(bByteNr)       Then bIBusError.1 = 1  '2
        'If iBusFrame(1) <> 4                        Then bIBusError.2 = 1  '4
        'If iBusFrame(1) = 5                        Then bIBusError.3 = 1  '8
        If iBusFrame(2) < 0x81                     Then bIBusError.4 = 1  '16
        If iBusFrame(2) > 0xAF                     Then bIBusError.5 = 1  '32
      Else                      'if frame-length value is too high ...
        bIBusError = 0xFF       'set errorflag
        Repeat 80 : End Repeat
    End If

  Loop Until bIBusError = 0   'end receive if no errors

End Sub


Sub iBusSend()
; Sends one iBus dataframe and calculates wChecksum:

  Dim wChecksum As Word
  wChecksum = 0xFFFF
  Dim bByteNr As Byte

  ;Set pin direction to output and activate stopbit-level:
  Set RECEIVER_PORT.RECEIVER_PIN On
  Dir RECEIVER_PORT.RECEIVER_PIN Out

  ;If sensor connected to another sensor 500us delay is needed:
  wait 60 10us  '500 microSeconds

  ;Send an build checksum on the fly:
  For bByteNr = 1 To iBusFrame(1) - 2  'byte1 is the length of dataframe
    Ser2Send iBusFrame(bByteNr)        'send byte
    wChecksum -= iBusFrame(bByteNr)    'calculate wChecksum
  Next bByteNr

  Ser2Send wChecksum                   'send lowbyte
  Ser2Send wChecksum_H                 'send highbyte

  ;Set pin direction back to input and activate stopbit-level:
  Set RECEIVER_PORT.RECEIVER_PIN On
  Dir RECEIVER_PORT.RECEIVER_PIN In

End Sub
